/*
	UScanIT project
	clam DB reading (http://www.clamav.net/)
	Rmi THOMAS, May 2004, remi@xtware.com
*/

#include "StdAfx.h"
#include ".\clamdb.h"
#include "Tools.h"

clam::clamDB::~clamDB(void)
{
	m_files.Destroy();
}

// This is TAR GZ file
bool clam::clamDB::Load(const char * fileName)
{
	bool bResult = false;

	FILE *file = fopen(fileName, "rb");
	if (file)
	{
		// Go over header
		fseek(file, 512, SEEK_SET);
		gzFile cfile = gzdopen(fileno(file), "rb");
		if (cfile)
		{
			int size=0;
			bool inBlock=false;
			clamFile *currentFile=0;

			char buffer[512];
			int read;
			do
			{
				read = gzread(cfile, buffer, sizeof(buffer));
				if (inBlock)
				{
					if (currentFile)
					{
						int i = min(read, size);
						currentFile->m_data.Write(buffer, i);
						size -= i;
						if (size==0)
							inBlock=false;
					}
				}
				else
				{
					// If filename is empty then exit
					if (buffer[0]==0)
						break;
					if (currentFile)
					{
						currentFile->m_data.Seek(0);
						m_files.AddTail(currentFile);
					}
					currentFile = new clamFile;
					currentFile->m_name = buffer;
					char *pt = buffer+124;
					sscanf(pt, "%o", &size);
					inBlock=true;
				}
			}
			while (read==sizeof(buffer));

			if (currentFile)
			{
				currentFile->m_data.Seek(0);
				m_files.AddTail(currentFile);
				bResult=true;
			}

			gzclose(cfile);
		}
	}
	return bResult;
}

bool clam::clamDB::ReadDBInfo (pixtek::tools::ClassStream *in, clamDBInfo *out)
{
	bool bResult=false;

	char buffer[513];
	int read = in->Read(buffer, 512);
	if (read==512)
	{
		buffer[512]=0;
		std::string header(buffer);
		Tools::Trim(header);

		std::vector<std::string> parsedHeader;
		int fields = Tools::Split(':', header, parsedHeader);
		if (fields>=8)
		{
			out->m_header				= parsedHeader[0];
			out->m_date					= parsedHeader[1];
			out->m_version				= atol(parsedHeader[2].c_str());
			out->m_numberOfSignature	= atol(parsedHeader[3].c_str());
			out->m_functionnalityLevel  = atol(parsedHeader[4].c_str());
			out->m_MD5					= parsedHeader[5];
			out->m_digitalSignature		= parsedHeader[6];
			out->m_builder				= parsedHeader[7];
			bResult = true;
		}
	}

	return bResult;
}

bool clam::clamDB::ReadSignature(pixtek::tools::ClassStream *in, ClassLinkList2<clamSignature> *out)
{
//	unsigned char hexabuffer[2048];
	unsigned short hexabuffer[2048];
	unsigned char hexaconv1[256]={}, hexaconv2[256];
	for (int i='0'; i<='9'; i++)
		hexaconv1[i]=i-'0';
	for (int i='a'; i<='f'; i++)
		hexaconv1[i]=i-'a'+10;
	for (int i='A'; i<='F'; i++)
		hexaconv1[i]=i-'A'+10;
	for (int i=0; i<256; i++)
		hexaconv2[i]=hexaconv1[i]<<4;

	bool bResult=false;
	int nodes = out->m_iTotalNode;
	int size = in->Size();
	if (size>0)
	{
		std::string sig;
		char buffer[512];
		int rsize;
		while ((rsize = in->Read(buffer, sizeof(buffer)-1))>0)
		{
			buffer[rsize]=0;
			sig.append(buffer);
		}
		std::vector<std::string> list;
		int virus = Tools::Split(0x0A, sig, list);
		if (virus>0)
		{
			std::vector<std::string>::iterator pos = list.begin();
			while (pos!=list.end())
			{
				std::vector<std::string> parts;
				if (Tools::Split('=', *pos, parts)==2)
				{
					clamSignature *signature = new clamSignature;
					strncpy(signature->m_virusName, parts[0].c_str(), sizeof(signature->m_virusName)-1);

					std::vector<std::string> sigParts;
					Tools::Split('*', parts[1], sigParts);
					std::vector<std::string>::iterator posSig = sigParts.begin();
					while (posSig!=sigParts.end())
					{
						pixtek::tools::MemBlob *blob = new pixtek::tools::MemBlob;
						// Convert hexa to binary
						int idx=0;
						const char *pt = (*posSig).c_str();
						while (*pt)
						{
							if (*pt=='?')
							{
								hexabuffer[idx++] = 256;
								pt++;
							} else
							{
								hexabuffer[idx] = hexaconv2[*pt++];
								hexabuffer[idx++] += hexaconv1[*pt++];
							}
						}
						if (idx>0)
						{
							if (blob->Init(idx*2))
								memcpy(blob->GetData(), hexabuffer, idx*2);
						}
						if (blob->GetSize()>0)
							signature->m_data.AddTail(blob);
						else
							delete blob;
						posSig++;
					}
					out->AddTail(signature);
				}
				pos++;
			}
			bResult=(out->m_iTotalNode>nodes);
		}
	}
	return bResult;
}

bool clam::clamDB::ComputeMD5 (const char *CVDfileName, std::string &out)
{
	bool					bResult=false;
	pixtek::tools::FileBlob fb;
	if (fb.MapFile(CVDfileName))
	{
		CMD5Checksum			md5;
		pixtek::tools::MemBlob	result;

		md5.Update((BYTE *)(((char *)fb.GetData())+512), fb.GetSize()-512);
		md5.Final(&result);
		char buffer[128];
		result.ToHexa(buffer, sizeof(buffer));
		out = buffer;
		bResult=true;
	}
	return bResult;
}

bool clam::clamDB::ParseDate (std::string &date, SYSTEMTIME *time)
{
	const char *mnames = "JanFebMarAprMayJunJulAugSepOctNovDec";

	bool bResult=false;

	ZeroMemory(time, sizeof(SYSTEMTIME));
	char month[16], tz[16];
	int day, year, hour, min;

	if (sscanf(date.c_str(), "%d %s %d %d-%d %s", &day, &month, &year, &hour, &min, &tz)==6)
	{
		char *pt = strstr(mnames, month);
		if (pt!=NULL)
		{
			time->wMonth = (int)((pt-mnames)/3);
			time->wDay = day;
			time->wYear = year;
			time->wHour = hour;
			time->wMinute = min;
			bResult=true;
		}
	}
	return bResult;
}

bool clam::clamDB::ReadWebDBInfo (const char *fileToGet, clamDBInfo *out)
{
	bool bResult=false;
	ClassInternet ci;
	CStr url;
	// Add range 
	ci.HeaderAdd("Range: bytes=0-511");
	url.Format("http://database.clamav.net/%s", fileToGet);
	if (ci.GetWithHeaders(url.SafeGet()))
	{
		char buffer[512];
		pixtek::tools::MemoryStream ms;
		int size = ci.GetAnswerSize();
		while (size>0)
		{
			int read = ci.GetAnswer(buffer, size);
			ms.Write(buffer, read);
			size -= read;
		}
		ms.Seek(0);
		bResult = ReadDBInfo(&ms, out);
	}

	return bResult;
}

bool clam::clamDB::ReadWebDB (const char *fileToGet, pixtek::tools::MemoryStream *out)
{
	bool bResult=false;
	ClassInternet ci;
	CStr url;
	// Add range 
	url.Format("http://database.clamav.net/%s", fileToGet);
	if (ci.GetWithHeaders(url.SafeGet()))
	{
		char buffer[512];
		int size = ci.GetAnswerSize();
		if (size>0)
		{
			while (size>0)
			{
				int read = ci.GetAnswer(buffer, min(size, sizeof(buffer)));
				out->Write(buffer, read);
				size -= read;
			}
			bResult=true;
		}
	}

	return bResult;
}

// Returns 1 if date0>date1, returns 0 if date0==date1, returns -1 if date1>date0
int clam::clamDB::CompareDate (std::string &date0, std::string &date1)
{
	int result=0;

	SYSTEMTIME sTime0, sTime1;
	if (ParseDate(date0, &sTime0) && ParseDate(date1, &sTime1))
	{
		LONGLONG time0, time1;
		SystemTimeToFileTime(&sTime0, (LPFILETIME)&time0);
		SystemTimeToFileTime(&sTime1, (LPFILETIME)&time1);
		if (time0>time1)
			result=1;
		else if (time1>time0)
			result=-1;
	}

	return result;
}