/*
	UScanIT project
	clam DB reading (http://www.clamav.net/)
	Rmi THOMAS, May 2004, remi@xtware.com
*/
#pragma once

#include <ClassStream.h>
#include <string>
#include <vector>

namespace clam
{
	struct clamFile
	{
		pixtek::tools::MemoryStream		m_data;
		std::string						m_name;
	};

	struct clamDBInfo
	{
		std::string						m_header;
		std::string						m_date;
		int								m_version;
		int								m_numberOfSignature;
		int								m_functionnalityLevel;
		std::string						m_MD5;
		std::string						m_digitalSignature;
		std::string						m_builder;
	};

	class clamScanner;

	class clamSignature
	{
		friend class clamScanner;
		friend class clamSuperNode;
		int	idDocument;	// internal use for multi-parts signature
		int	magicCode;	// internal use for multi-parts signature (1 bit/part)
	public:
		virtual ~clamSignature()						{ m_data.Destroy(); }

		ClassLinkList2<pixtek::tools::MemBlob>			m_data;			// Each virus part
																		// This is a unsigned char array with escape caracters
																		// 0,0 mean value = 0
																		// 0,1 mean value is a joker (any value)
		char											m_virusName[64];
	};

	class clamDB
	{
		ClassLinkList2<clamFile>		m_files;

	public:
		virtual ~clamDB(void);

				bool		Load		(const char * fileName);
				int			FileCount	()					{ return m_files.m_iTotalNode; }
				clamFile *	GetFile		(int index)			{ return m_files[index]; }

		static	bool		ReadDBInfo	(pixtek::tools::ClassStream *in, clamDBInfo *out);
		static	bool		ReadSignature(pixtek::tools::ClassStream *in, ClassLinkList2<clamSignature> *out);
		static	bool		ComputeMD5	(const char *CVDfileName, std::string &out);
		static	bool		ParseDate	(std::string &date, SYSTEMTIME *time);

		// Web update
		static	bool		ReadWebDBInfo	(const char *fileToGet, clamDBInfo *out);
		static	bool		ReadWebDB		(const char *fileToGet, pixtek::tools::MemoryStream *out);
		// Returns 1 id date0>date1, returns if date0==date1, returns -1 if date1>date0
		static	int			CompareDate		(std::string &date0, std::string &date1);
	};
};